#!/bin/bash

# -------------------------------------------------------------------------- #
# Licensed under the Apache License, Version 2.0 (the "License"); you may    #
# not use this file except in compliance with the License. You may obtain    #
# a copy of the License at                                                   #
#                                                                            #
# http://www.apache.org/licenses/LICENSE-2.0                                 #
#                                                                            #
# Unless required by applicable law or agreed to in writing, software        #
# distributed under the License is distributed on an "AS IS" BASIS,          #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   #
# See the License for the specific language governing permissions and        #
# limitations under the License.                                             #
#--------------------------------------------------------------------------- #

##############################################################################
## START EDITING HERE
##############################################################################

sitename=""            # Usually a short provider name, e.g.
                       #   CESNET

cloudtype="OpenNebula"

occi_endpoint_url=""   # URL of your OCCI endpoint, e.g.
                       #   https://fqdn.example.com:80
                       # Always without the trailing slash!

mysql="/usr/bin/mysql" # Information necessary for direct DB access
mysql_user=""
mysql_password=""
mysql_host=""
mysql_db=""

# Recent SSM update imposed some restrictions on filenames
# for outgoing messages:
#
#  /var/spool/apel/outgoing/<8_HEX_DIGITS>/<14_HEX_DIGITS>
#
# e.g., if you want to save the message as
#
#  /var/spool/apel/outgoing/00000000/00000000000001
#
# keep the apel_outgoing_dir variable empty, since
#
#  /var/spool/apel/outgoing/00000000
#
# is the default value.

apel_outgoing_dir=""

##############################################################################
## STOP EDITING HERE
##############################################################################

# Run an SQL query on the database
get_sql() {
  $mysql -s -h $mysql_host -u $mysql_user -p$mysql_password $mysql_db -e "$1"

  if [ "$?" != "0" ]; then
    echo "CloudAcc couldn't get data from MySQL DB '$mysql_db' at '$mysql_host' as user '$mysql_user'!"
    exit 2
  fi
}

# No apel_outgoing_dir given, using the default value
if [ "$apel_outgoing_dir" = "" ]; then
  apel_outgoing_dir="/var/spool/apel/outgoing/00000000"
fi

# Make sure the directory exists
mkdir -p $apel_outgoing_dir

if [ "$?" != "0" ]; then
  echo "CloudAcc doesn't have permissions necessary to access and write into '$apel_outgoing_dir'!"
  exit 1
fi

# Define state id => SSM state mapping
declare -A states

states[0]="started"             # INIT
states[1]="started"             # PENDING
states[2]="suspended"		# HOLD
states[3]="started"             # ACTIVE
states[4]="suspended"		# STOPPED
states[5]="suspended"		# SUSPENDED
states[6]="completed"		# DONE
states[7]="completed"		# FAILED
states[8]="suspended"           # POWEROFF

# Read all user details for faster lookup later on
declare -A users
declare -A users_auth
declare -A users_dn

while read id user_xml
do

  # Get user's auth driver
  [[ $user_xml =~ \<AUTH_DRIVER\>(.*)\</AUTH_DRIVER\> ]]
  users_auth["$id"]=${BASH_REMATCH[1]}

  # Check for X509_DN attribute in user's template
  [[ $user_xml =~ \<X509_DN\>\<\!\[CDATA\[([^\<]*)\]\]\>\</X509_DN\> ]]
  if [ "${users_auth["$id"]}" = "x509" ] && [ "${BASH_REMATCH[1]}" != "" ]; then
    users_dn["$id"]="${BASH_REMATCH[1]}"
  else
    users_dn["$id"]=NULL
  fi

done < <(get_sql "SELECT oid, body FROM user_pool;")

# We have to count records to keep the message file size down;
# APEL/SSM devs recommend about 1k records per file.
message_num=0

# Construct a message for each VM
while read vm_id user_id group_id vm_name
do

  if [ "$vm_id" = "" ] || [ "$vm_name" = "" ]; then
    echo "Skipping a malformed record! VM_ID and VM_NAME are mandatory!"
    continue
  fi

  echo "Processing vm-id: $vm_id ($vm_name)"

  # Get VM XML description
  vm_xml=$(get_sql "SELECT CONCAT('\'', body, '\'') FROM vm_pool WHERE oid=$vm_id;")

  if [ "$vm_xml" = "" ]; then
    echo "Skipping a malformed record! VM has to have a BODY (XML)!"
    continue
  fi

  [[ $vm_xml =~ ^.*\<STATE\>([0-9]+)\</STATE\>.*$ ]]
  state="${BASH_REMATCH[1]}"

  # StartTime
  [[ $vm_xml =~ \<STIME\>([0-9]+)\</STIME\> ]]
  stime="${BASH_REMATCH[1]}"

  if [ "$stime" = "" ] || [ "$stime" = "0" ]; then
    echo "Skipping a malformed record! VM has to have a StartTime!"
    continue
  fi

  # OCCI ID
  [[ $vm_xml =~ \<OCCI_ID\>\<\!\[CDATA\[([^\<]*)\]\]\>\</OCCI_ID\> ]]
  occi_id="${BASH_REMATCH[1]}"

  if [ "$occi_id" = "" ]; then
    echo "Skipping a record without an OCCI ID present!"
    continue
  fi

  # Did we reach another 1k of messages?
  if [ "$((message_num%1000))" -eq "0" ]; then
    # Get a sequence number for the new file from the
    # number of processed messages
    file_num=$((message_num/1000+1))

    # Pad the number with zeros, set the apel_outgoing_file variable
    # and insert a new file header
    apel_outgoing_file="$apel_outgoing_dir/`printf %014d $file_num`"

    echo "APEL-cloud-message: v0.2" > $apel_outgoing_file

    if [ "$?" != "0" ]; then
      echo "CloudAcc couldn't create and write into file '$apel_outgoing_file'!"
      exit 3
    fi
  fi

  # RecordId
  echo "VMUUID: $occi_endpoint_url/compute/$occi_id $(date --date=@$stime -u --rfc-3339=seconds)" >> $apel_outgoing_file

  # SiteName
  echo "SiteName: $sitename" >> $apel_outgoing_file

  # MachineName
  echo "MachineName: one-$vm_id" >> $apel_outgoing_file

  # LocalUserId
  if [ "$user_id" = "" ]; then
    user_id=NULL
  fi
  echo "LocalUserId: $user_id" >> $apel_outgoing_file

  # LocalGroupId
  if [ "$group_id" = "" ]; then
    group_id=NULL
  fi
  echo "LocalGroupId: $group_id" >> $apel_outgoing_file

  # GlobalUserName or NULL, if DN is not available
  if [ "${users_dn["$user_id"]}" = "" ]; then
    users_dn["$user_id"]=NULL
  fi
  echo "GlobalUserName: ${users_dn["$user_id"]}" >> $apel_outgoing_file

  # FQAN
  [[ $vm_xml =~ \<GNAME\>(.*)\</GNAME\> ]]
  fqan=${BASH_REMATCH[1]}

  if [ "$fqan" = "" ]; then
    echo "FQAN: NULL" >> $apel_outgoing_file
  else
    echo "FQAN: /$fqan/Role=NULL/Capability=NULL" >> $apel_outgoing_file
  fi

  # Status
  [[ $vm_xml =~ ^.*\<STATE\>([0-9]+)\</STATE\>.*$ ]]
  state_id="${BASH_REMATCH[1]}"
  state=${states["$state_id"]}

  if [ "$state" = "" ]; then
    state=NULL
  fi
  echo "Status: $state" >> $apel_outgoing_file

  # StartTime, already validated and used in RecordId
  echo "StartTime: $stime" >> $apel_outgoing_file

  # EndTime
  [[ $vm_xml =~ \<ETIME\>([0-9]+)\</ETIME\> ]]
  etime="${BASH_REMATCH[1]}"

  if [ "$etime" = "0" ] || [ "$etime" = "" ]; then
    # VM still running => use NULL as EndTime
    etime=NULL
  fi
  echo "EndTime: $etime" >> $apel_outgoing_file

  # SuspendDuration
  echo "SuspendDuration: NULL" >> $apel_outgoing_file

  # WallDuration
  echo "WallDuration: NULL" >> $apel_outgoing_file

  # CpuDuration
  echo "CpuDuration: NULL" >> $apel_outgoing_file

  # CpuCount
  [[ $vm_xml =~ \<VCPU\>\<\!\[CDATA\[([0-9]+)\]\]\>\</VCPU\> ]]
  vcpu="${BASH_REMATCH[1]}"

  if [ "$vcpu" = "0" ] || [ "$vcpu" = "" ]; then
      # VM uses default, which is probably = 1 :)
      vcpu=1
  fi
  echo "CpuCount: $vcpu" >> $apel_outgoing_file

  # NetworkType
  echo "NetworkType: NULL" >> $apel_outgoing_file

  # NetworkInbound
  [[ $vm_xml =~ \<NET_RX\>([0-9]+)\</NET_RX\> ]]
  net_rx="${BASH_REMATCH[1]}"

  if [ "$net_rx" = "" ]; then
    net_rx=0
  fi
  net_rx=$((net_rx/1073741824))

  if [ "$((net_rx%1073741824))" -gt "536870911" ]; then
    net_rx=$((net_rx+1))
  fi
  echo "NetworkInbound: $net_rx" >> $apel_outgoing_file

  # NetworkOutbound
  [[ $vm_xml =~ \<NET_TX\>([0-9]+)\</NET_TX\> ]]
  net_tx="${BASH_REMATCH[1]}"

  if [ "$net_tx" = "" ]; then
    net_tx=0
  fi
  net_tx=$((net_tx/1073741824))

  if [ "$((net_tx%1073741824))" -gt "536870911" ]; then
    net_tx=$((net_tx+1))
  fi
  echo "NetworkOutbound: $net_tx" >> $apel_outgoing_file

  # Memory
  [[ $vm_xml =~ \<MEMORY\>\<\!\[CDATA\[([0-9]+)\]\]\>\</MEMORY\> ]]
  mem="${BASH_REMATCH[1]}"

  if [ "$mem" = "" ]; then
    mem=0
  fi
  echo "Memory: $mem" >> $apel_outgoing_file

  # Disk
  echo "Disk: NULL" >> $apel_outgoing_file

  # StorageRecordId
  echo "StorageRecordId: NULL" >> $apel_outgoing_file

  # ImageId, using name of the image from OpenNebula
  #[[ $vm_xml =~ \<IMAGE_ID\>\<\!\[CDATA\[([0-9]+)\]\]\>\</IMAGE_ID\> ]]
  [[ $vm_xml =~ \<IMAGE\>\<\!\[CDATA\[([^\<]*)\]\]\>\</IMAGE\> ]]
  image="${BASH_REMATCH[1]}"

  if [ "$image" = "" ]; then
    image=NULL
  fi
  echo "ImageId: $image" >> $apel_outgoing_file

  # CloudType
  echo "CloudType: $cloudtype" >> $apel_outgoing_file

  echo "%%" >> $apel_outgoing_file

  # Increment message_num
  message_num=$((message_num+1))

done < <(get_sql "SELECT oid, uid, gid, CONCAT('\'', name, '\'') FROM vm_pool;")
